@@ -5,4 +5,7 @@ from arc.defaults import *
 from arc.collections import *
 from arc.responses import *
 from arc.templates import *
-from arc.sessions import *
\ No newline at end of file
+from arc.sessions import *
+from arc.forms import *
+
+__version__ = "2.5.8"
\ No newline at end of file
