@@ -1,9 +1,7 @@
-from re import L
 from arc.middleware import Middleware
 from arc.defaults import LoggingMiddleware, DefaultExceptionHandler
 from arc.staticfiles import StaticFile
 from arc.logger import Logger
-from starlette.websockets import WebSocket
 import uvicorn
 from parse import parse
 import traceback
@@ -84,7 +82,7 @@ class App:
         if exception_handler is None:
             self.exception_handler = DefaultExceptionHandler(self)
         else:
-            self.exception_handler = exception_handler()
+            self.exception_handler = exception_handler(self)
 
         self.middleware = Middleware(self)
 
@@ -101,6 +99,11 @@ class App:
 
         self.config = {}
 
+        self.before_request_funcs = []
+
+        self.methods = ["GET", "POST", "HEAD", "PUT",
+                        "DELETE", "CONNECT", "OPTIONS", "TRACE", "PATCH"]
+
     async def __call__(self, scope, receive, send):
         self.scope = scope
 
@@ -108,6 +111,12 @@ class App:
 
     async def handle_request(self, request, type):
         try:
+            for func in self.before_request_funcs:
+                if inspect.iscoroutinefunction(func):
+                    await func()
+                else:
+                    func()
+
             handler, kwargs, methods = self.find_handler(
                 request_path=request.url.path)
 
@@ -123,26 +132,22 @@ class App:
                     return self.default_response
 
                 return
-            
+
             elif type == "http":
-                assert request.method.lower() in [method.lower(
-                ) for method in methods], f"Method {request.method.lower()} not allowed"
+                if methods is not None:
+                    assert request.method.lower() in [method.lower(
+                    ) for method in methods], f"Method {request.method.lower()} not allowed"
 
-            try:
-                if handler is not None:
+            if handler is not None:
 
-                    if inspect.iscoroutinefunction(handler):
-                        response = await handler(request, **kwargs)
+                if inspect.iscoroutinefunction(handler):
+                    response = await handler(request, **kwargs)
 
-                    else:
-                        response = handler(request, **kwargs)
                 else:
-                    return self.default_response(request)
-
-            except Exception as e:
-                error = traceback.format_exc()
-                print(error)
-                response = self.exception_handler.handle_error(request, error)
+                    response = handler(request, **kwargs)
+            else:
+                return self.default_response(request)
+            
         except:
             error = traceback.format_exc()
             print(error)
@@ -155,14 +160,21 @@ class App:
 
     def find_handler(self, request_path):
         for path, items in self.routes.items():
-            handler = items["handler"]
-            try:
-                methods = items["methods"]
-            except:
-                methods = None
             parse_result = parse(path, request_path)
-            if parse_result is not None:
-                return handler, parse_result.named, methods
+            if isinstance(items, dict):
+                handler = items["handler"]
+                try:
+                    methods = items["methods"]
+                except:
+                    methods = None
+
+                if parse_result is not None:
+                    return handler, parse_result.named, methods
+            else:
+                handler = items
+                methods = self.methods
+                if parse_result is not None:
+                    return handler, parse_result.named, methods
 
         return None, None, methods
 
@@ -186,6 +198,19 @@ class App:
         self.routes[path] = {"handler": handler,
                              "methods": methods}
 
+    def add_routes(self, routes):
+        for path, items in routes.items():
+            assert path not in self.routes, f"Route {path} already exists"
+
+            try:
+                methods = items["methods"]
+            except KeyError:
+                methods = ["GET", "POST", "HEAD", "PUT", "DELETE",
+                           "CONNECT", "OPTIONS", "TRACE", "PATCH"]
+
+            self.routes[path] = {
+                "handler": items["handler"], "methods": methods}
+
     def websocket(self, path: str):
         assert path not in self.routes, f"Route {path} already exists"
 
@@ -200,6 +225,14 @@ class App:
 
         self.routes[path] = {"handler": handler}
 
+    def before_request(self, handler):
+
+        def wrapper(handler):
+            self.before_request_funcs.append(handler)
+            return handler
+
+        return handler
+
     def add_middleware(self, middleware_cls, *args):
         self.middleware.add(middleware_cls, *args)
 
